
//**********************************************
//
//  IR  
//   Marian Kulesza 
// email: kulemar@gmail.com
// http://sites.google.com/site/mariankmh/
//
//   : 
//     IR      
//
// IR       INT1,        .
//      1,      .
//   ,      .
//    (,    8000  * 23.15  = 0.1852 )
//      IR_buffer,     .
//
//**********************************************

// cpu: ATMega16
// speed: 11.0592 MHz

#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#define buffer_size	    255		// ,     ,   ,
								//   PulseNum  "unsigned char"  "unsigned int"
								// ,  RAM  . 

//    overflow_delay  0xFFFF
//     timera1 (0xFFFF - E0BF = 0x1F40) (0x1F40  8000 )
//overflow_delay   >= 0x8000  < 0xFFFF

//           11059200 Hz / 256 = 43200  Hz
//               1 / 43200 = 23.15  
//,    8000  * 23.15  = 0.1852 

#define overflow_delay	0xE0BF 				
#define last_impuls  	0x7FFF 							//     0x7FFF  0x8000+0x7FFF=0xFFFF 
#define timer1_stop		(1<<CS12)|(1<<CS11)|(1<<CS10)	//timer1 stop
#define timer1_run		(1<<CS12)						//clk/256

// 
static unsigned int  IR_buffer [buffer_size];

static unsigned char PulseNum = 0;  //  
static unsigned char IRpin    = 0;  //  IR 
static unsigned int	 Timer    = 0;  //   
static unsigned char Poslac   = 1;  // 1 -  , 0 -      

// UART
static int uart_putchar(char c, FILE *stream);
static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL,_FDEV_SETUP_WRITE);

static int uart_putchar(char c, FILE *stream)
{
   loop_until_bit_is_set(UCSRA, UDRE);
   UDR = c;
   return 0;
}

void ostanowka_timera()
{
	GICR  = 0;	   // INT1
	
	TCCR1B &= ~(timer1_stop);		//  1

	//    
	TCNT1=overflow_delay;	//  overflow_delay  0xFFFF
	

	if (PulseNum != 0)
	{        
		IR_buffer[PulseNum-1]+=last_impuls; //  Timer 0,     
		Poslac = 0; // IR 
	}
}

ISR(TIMER1_OVF_vect) // Timer/Counter1 Overflow
{
	ostanowka_timera();
}

// ----------------------------------------------------------------------------
// irq code (ext: INT1)
// External Interrupt Request 1
// ----------------------------------------------------------------------------

ISR(INT1_vect) // ext
{
	IRpin = (PIND & 0x08);
	Timer = TCNT1; 						//   1
	Timer = Timer - overflow_delay; 	//   (  Timer      )
  
	TCNT1=overflow_delay;				//   .   overflow_delay  0xFFFF

	if (PulseNum==0) TCCR1B = timer1_run; // CLK/256      Systemclock/256
               
	if (IRpin == 0x08)
	{
		PORTD |=(1<<7);				//PIND.7 = 1; //  
		IR_buffer[PulseNum]=0x0000;  	//   0,     0,   0
	} 
	else
	{		
		PORTD &= ~ (1<<7); 			//PIND.7 = 0; //  
		IR_buffer[PulseNum]=0x8000;  	//   1,     1,   0
	}
	if (PulseNum != 0) IR_buffer[PulseNum-1]+=Timer;   //   

	++PulseNum; 
	if (PulseNum >= (buffer_size-1)) ostanowka_timera();  //                   
}


void init()
{
	// usart '0'
	// settings: 57600 8-n-1
	UCSRA = 0;
	UCSRB = (1<<TXEN);
	UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);
	UBRRH = 0;
	UBRRL = 11;
	
    stdout = &mystdout;//     UART

	// port 'D'
	DDRD  = 0x80;	// 0=In  1=In  2=In  3=In  4=In  5=In  6=In  7=Out 
	PORTD = 0x88;	// 0=Low 1=Low 2=Low 3=Hi  4=Low 5=Low 6=Low 7=Hi

	// --- INT1 irq ---
	MCUCR |= (1<<ISC10); // Any logical change on INT1 generates an interrupt request
	GICR  |= (1<<INT1); // External Interrupt Request 1 Enable

	
	TIMSK = (1 << TOIE1); // Timer/Counter1 Overflow Interrupt Enable

	TCNT1=overflow_delay;	//  overflow_delay  0xFFFF

    PulseNum  = 0;     //  
	Poslac    = 1;

}

//-------------------------------------------------------------------------------------------------

void print_IR_buffer()
{
  unsigned char k;
  printf("s "); 					//poslac znak naczala posylki s = start
  for(k=0;k<PulseNum;k++)
  {
    printf("%u ",IR_buffer[k]);
  }
  printf("e");  					//poslac znak konca posylki   e = end
  printf("\n\r");
}


// ----------------------------------------------------------------------------
// main
// ----------------------------------------------------------------------------

int main()
{
	init();
	sei();

// --- main loop ---

	for (;;) 
	{
		if (Poslac == 0)  //ustanovitsa v nol, pri perepolnenii taimera
		{
			IRpin = (PIND & 0x08);
			if (IRpin == 0x08)
				PORTD |= (1<<7); //PIND.7 = 1; // potushit sveto diod
			else
				PORTD &= ~(1<<7);//PIND.7 = 0; // zazhech sveto diod
			if (PulseNum > 1) print_IR_buffer();

			Poslac = 1;     //zdem sled. perepolnenija taimera( posle prijema IR posledowatelnosti )
			PulseNum  =0;   //  
			GICR   |=0x80;  //  
		} //end poslac
	}
  return 0;
}

// ----------------------------------------------------------------------------

